﻿using System;
using System.Reflection;
using System.Windows.Forms;
using VTableNamespace;
using Word = Microsoft.Office.Interop.Word;

namespace TableStructurer
{
    /// <summary>Klasa okna glownego aplikacji TableStructurer</summary>
    public partial class FormMain : Form
    {
        /// <summary>
        /// Pole przechowujace nazwe otwieranego dokumentu Word
        /// wraz ze sciezka polozenia w drzewie katalogow
        /// </summary>
        private string documentPath;

        /// <summary>Pole przechowujace obiekt klasy VTable zawierajacy dane o naglowku dokumntu Word</summary>
        public VTable THeader;

        /// <summary>Pole przechowujace obiekt klasy VTable zawierajacy macierz pozycjonujaca</summary>
        public VTable TMatrix;

        /// <summary>Pole przechowujace obiekt klasy VTable zawierajacy tabele z dokumentu</summary>
        private VTable TTable;

        /// <summary>Konstruktor okna glownego aplikacji TableStructurer</summary>
        public FormMain()
        {
            InitializeComponent();
            btnOrder.Dock = DockStyle.Left;
            btnClose.Dock = DockStyle.Left;
            btnOrder.Visible = true;
            btnClose.Visible = false;
            documentPath = Application.StartupPath + "\\DokumentWord.docx";
            btnOrder.Click += new EventHandler(btnOrder_Click);
            btnClose.Click += new EventHandler(btnClose_Click);
        }

        /// <summary>Metoda obslugi zdarzenia Click przycisku Porzadkuj</summary>
        /// <param name="sender">Przycisk Porzadkuj</param>
        /// <param name="e">Parametry zdarzenia</param>
        private void btnOrder_Click(object sender, EventArgs e)
        {
            // zamykanie procesow Word'a przydatne na etapie testowania kodu
            // Utilities.CloseWordProcesses();

            Word.Application word = new Word.Application();

            object oPath = documentPath;
            object oMissing = Missing.Value;
            object oReadOnly = true;
            
            Word.Document doc;

            try
            {
                doc = word.Documents.Open(ref oPath, ref oMissing, ref oReadOnly);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);

                // ponieważ metoda Quit daje warning niejednoznacznośći (ang. ambiguity) Quit
                // używamy rzutowania na Word._Application
                
                (word as Word._Application).Quit();
                return;
            }

            if (TableHeader.GetHeader(word, doc, ref THeader, ref TMatrix, gridHeader, gridMatrix))
            {
                TTable = new VTable(THeader.ColumnCount, gridTable);

                if (HtmlToVTable.Fill(TTable, word, doc))
                {
                    
                    TTable.ShiftCells(TMatrix);
                    THeader.Show(VTypes.Rec);
                    TMatrix.Show(VTypes.Int);
                    TTable.Show(VTypes.Rec);
                }
            }
            else
            {
                doc.Select();
                word.ActiveWindow.Close();
            }

            try
            {
                // na wypadek gdyby dokument nie został zamknięty przez HTMLToGrid.Fill

                doc.Select();
                word.ActiveWindow.Close();
            }
            catch { /* dokument został zamknięty wcześniej */ }

            btnOrder.Visible = false;
            btnClose.Visible = true;
        }

        /// <summary>Metoda obslugujaca zdarznie Click przycisku Zamknij</summary>
        /// <param name="sender">Przycisk Zamknij</param>
        /// <param name="e">Parametry zdarzenia</param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
