﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

namespace TableStructurer
{
    /// <summary>
    /// Klasa zawierajaca parser kodu HTML
    /// </summary>
    public class HtmlParser
    {
        /// <summary>
        /// Metoda pobierajaca wartosc tagu body w dokumencie HTML,
        /// na poczatku metoda wczytuje dokument z pliku do zmiennej string 
        /// o nazwie html
        /// </summary>
        /// <param name="filename">
        /// Nazwa dokumentu wraz ze sciezka polozenia w drzewie katalogow
        /// </param>
        /// <param name="body">Parametr zwracajacy wartosc tagu body w formie string</param>
        /// <returns>True w przypadku powodzenia, False w przypadku niepowodzenia metody</returns>
        public static bool GetBody(string filename, out string body)
        {
            bool Result = true;
            string html = "";
            FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read);
            StreamReader sr = new StreamReader(fs, System.Text.Encoding.GetEncoding(1250));

            try
            {
                html = sr.ReadToEnd();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                Result = false;
            }
            finally
            {
                sr.Close();
            }

            body = html.Substring(html.IndexOf("<body") + ("<body").Length);
            body = body.Substring(body.IndexOf(">") + 1);
            body = body.Substring(0, body.IndexOf("</body>"));
            return Result;
        }

        /// <summary>
        /// Metoda sprawdzajaca czy komorki tabel dokumetu HTML zawieraja scalenia w pionie
        /// </summary>
        /// <param name="body">Wartosc tagu body dokumentu html</param>
        /// <returns>True jesli znalezono string rowspan, False jesli nie znaleziono</returns>
        public static bool ContainsVertMerge(string body)
        {
            bool RowSpan = false;

            if (body.IndexOf("<table") >= 0)
            {
                string tables = body.Substring(body.IndexOf("<table") + ("<table").Length);

                if (tables.LastIndexOf("</table>") >= 0)
                {
                    tables = tables.Substring(0, tables.LastIndexOf("</table>"));
                    RowSpan = tables.Contains("rowspan=");
                }
            }

            return RowSpan;
        }

        /// <summary>Metoda pobierajaca warotosc tagu table dokumentu HTML</summary>
        /// <param name="body">Modyfikowana warosci tagu body</param>
        /// <param name="NextTableExists">
        /// Zwracany parametr typu bool, True jesli po danej tabeli jest nastepna tabela,
        /// False jesli nastepnej tabeli nie ma
        /// </param>
        /// <param name="NestedTable">
        /// Zwracany parametr bool, True jesli tabela jest zagniezdzona tj. zawiera inna tabele,
        /// False jesli tabela nie zawiera zagniezdzen
        /// </param>
        /// <returns>Wartosc tagu table</returns>
        public static string GetTable(ref string body, ref bool NextTableExists, ref bool NestedTable)
        {
            if (body.IndexOf("<table") >= 0)
            {
                string table = body.Substring(body.IndexOf("<table") + ("<table").Length);

                table = table.Substring(table.IndexOf(">") + 1);
                table = table.Substring(0, table.IndexOf("</table>"));

                if (table.Contains("<table"))
                {
                    NestedTable = true;
                    return "";
                }

                body = body.Substring(body.IndexOf("</table>") + ("</table>").Length);
                NextTableExists = body.IndexOf("<table") >= 0;
                return table;
            }
            else
            {
                NextTableExists = false;
                return "";
            }
        }

        /// <summary>Metoda pobierajaca wartosc tagu tr dokumentu HTML (wiersz tabeli)</summary>
        /// <param name="table">Wartosc tagu table</param>
        /// <param name="rows">Zwracana w ArrayList lista wierszy</param>
        public static void GetTableRows(string table, ArrayList rows)
        {
            string row;

            rows.Clear();

            while (table.IndexOf("<tr") >= 0)
            {
                row = table.Substring(table.IndexOf("<tr") + ("<tr").Length);
                row = row.Substring(row.IndexOf(">") + 1);
                row = row.Substring(0, row.IndexOf("</tr>"));
                rows.Add(row);
                table = table.Substring(table.IndexOf("<tr") + ("<tr").Length);
            }
        }

        /// <summary>Metoda pobierajaca wartosci tagow td (komorki wiersza tabeli)</summary>
        /// <param name="row">Wartosc tagu tr (wiersz tabeli)</param>
        /// <param name="cells">Zwracana w ArrayList lista wartosci tagow td (komorki wiersza)</param>
        /// <param name="widths">Zwracana lista szerokosci komorek tabeli</param>
        /// <returns>True w przypadku powodzenia, False w przypadku niepowodzenia</returns>
        public static bool GetRowCells(string row, ArrayList cells, List<int> widths)
        {
            string cell;

            cells.Clear();
            widths.Clear();

            while (row.IndexOf("<td") >= 0)
            {
                string width;
                int widthi;

                cell = row.Substring(row.IndexOf("<td") + ("<td").Length);
                width = cell.Substring(cell.IndexOf("style='width:") + ("style='width:").Length);
                width = width.Substring(0, width.IndexOf(";"));

                // http://www.asknumbers.com/LengthConversion.aspx
                // https://www.w3.org/Style/Examples/007/units.en.html
                // 1 centimeter = 28.346456699999997 point [print]

                if (width.Contains("cm"))
                {
                    width = width.Substring(0, width.IndexOf("cm"));
                    float mul = 28.346456699999997F;
                    width = width.Replace(".", ",");
                    widthi = (int)Math.Round(float.Parse(width) * mul, 0);
                }
                else if (width.Contains("pt"))
                {
                    width = width.Substring(0, width.IndexOf("pt"));
                    float mul = 1F;
                    width = width.Replace(".", ",");
                    widthi = (int)Math.Round(float.Parse(width) * mul, 0);
                }
                else
                {
                    MessageBox.Show("Nieropoznana jednostka szerokości komórki");
                    return false;
                }

                cell = cell.Substring(cell.IndexOf(">") + 1);
                cell = cell.Substring(0, cell.IndexOf("</td>"));

                // Przyjeto ze komorka nie zawiera tagow span, b, u, i
                
                // Sposob na pomijanie atrybutow tagu
                // Dla uproszczenia pokazano tylko jeden tag

                while (cell.IndexOf("<p") >= 0)
                {
                    int p = cell.IndexOf("<p");
                    string replaceTag = cell.Substring(p, cell.Substring(p).IndexOf(">") + 1);
                    cell = cell.Replace(replaceTag, "");
                }

                cell = cell.Replace("</p>", "").Replace("&nbsp;", " ");
                cells.Add(cell.Trim());
                widths.Add(widthi);
                row = row.Substring(row.IndexOf("<td") + ("<td").Length);
            }

            return true;
        }
    }
}