﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using VTableNamespace;
using Word = Microsoft.Office.Interop.Word;

namespace TableStructurer
{
    /// <summary>Klasa  do przekazywania watosci tagow z dokumentu HTML do obiektow VTable</summary>
    class HtmlToVTable
    {
        /// <summary>Metoda pobierajaca dane z dokumentu do obiektu klasy VTable</summary>
        /// <param name="vtable">Obiekt klasy VTable</param>
        /// <param name="aWord">Aplikacja MS Word wskazana przez referncje COM</param>
        /// <param name="aDoc">Dokument aplikacji MS Word</param>
        /// <returns>True w przypadku powodzenia metody, False w przypadku niepowodzenia metody</returns>
        public static bool Fill(VTable vtable, Word.Application aWord, Word.Document aDoc)
        {
            string documentPath = Application.StartupPath + "\\DokumentHTML.htm";
            object oFilteredHTML = Word.WdSaveFormat.wdFormatFilteredHTML;
            object oPath = documentPath;

            try
            {
                // Poniewaz dokument pobrano w formacie .docx
                // nalezy go zapisac jako HTML, tu: przefiltrowany

                aDoc.SaveAs(ref oPath, ref oFilteredHTML);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }
            finally
            {
                // jeżeli tu nie zamknie sie dokumentu w HTMLParser.GetBody(...) wystąpi wyjątek:
                // proces nie może uzyskać dostępu ponieważ dokument jest używany przez inny proces

                aDoc.Select();
                aWord.ActiveWindow.Close();
            }

            ArrayList Rows = new ArrayList();
            bool NextExists = true;

            string Body;

            if (!HtmlParser.GetBody(documentPath, out Body))
            {
                return false;
            }

            if (HtmlParser.ContainsVertMerge(Body))
            {
                MessageBox.Show("Dokument zawiera scalenie komórek tabeli w pionie");
                return false;
            }

            int TotalRowCount = 0;
            bool NestedTable = false;

            vtable.RowCount = 0;

            while (NextExists)
            {
                string Table = HtmlParser.GetTable(ref Body, ref NextExists, ref NestedTable);

                if (NestedTable)
                {
                    MessageBox.Show("Dokument zawiera tabele zagnieżdżoną");
                    return false;
                }

                HtmlParser.GetTableRows(Table, Rows);

                vtable.RowCount += Rows.Count;
                
                int rowIdx1 = 0;
                int rowIdx2 = 0;

                foreach (string r in Rows)
                {
                    ArrayList Cells = new ArrayList();
                    List<int> Widths = new List<int>();

                    if (!HtmlParser.GetRowCells(Rows[rowIdx1++].ToString(), Cells, Widths))
                    {
                        return false;
                    }

                    int col = 0;

                    foreach (string cell in Cells)
                    {
                        int width = Widths[col];

                        vtable[col, TotalRowCount + rowIdx2].AsStr = cell;
                        vtable[col, TotalRowCount + rowIdx2].AsInt = width;
                        col++;

                        if (col == vtable.ColumnCount)
                        {
                            break;
                        }
                    }

                    rowIdx2++;

                    if (rowIdx2 == Rows.Count)
                    {
                        break;
                    }
                }

                TotalRowCount += rowIdx2;
            }

            return true;
        }
    }
}
