﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace VTableNamespace
{
    /// <summary>
    /// Klasa VTable uzywana do przechowywania, przetwarzania 
    /// i prezentacji danych pobranych z tabel dokumentu Word
    /// </summary>
    public partial class VTable
    {
        /// <summary>Pole - lista tablic</summary>
        private List<Array> Rows;

        /// <summary>Pole - komponent DataGridView</summary>
        private DataGridView Grid;

        /// <summary>Pole - rozmiar tablicy</summary>
        private int Size;

        /// <summary>Konstruktor klasy VTable</summary>
        /// <param name="columnCount">Ilosc ilementow tablicy klasy Array</param>
        /// <param name="grid">Opcjonalny komponent DataGridView do prezentacji danych</param>
        public VTable(int columnCount, DataGridView grid = null)
        {
            Rows = new List<Array>();
            Grid = grid;
            
            if (Grid != null)
            {
                Grid.AllowUserToAddRows = false;
                Grid.AllowUserToDeleteRows = false;
            }
            
            Size = columnCount;
        }
        
        /// <summary>
        /// Wlasnosc umozliwiajaca zmiane wartosci elementu klasy Value
        /// lub jej pobranie, gdzie Value jest elementem tablicy klasy Array
        /// znajdujacym sie na w liscie tablic tej klasy
        /// </summary>
        /// <param name="colIdx">Indesks elementu tablicy</param>
        /// <param name="rowIdx">Indeks elementu listy</param>
        /// <returns>Element klasy Value</returns>
        public Value this[int colIdx, int rowIdx]
        {
            set 
            {
                if (rowIdx >= 0 && rowIdx < Rows.Count)
                {
                    Rows[rowIdx][colIdx] = value;
                } 
                else 
                {
                    throw new Exception(GetType().Name + " row index error");
                } 
            }
            
            get 
            {
                if (rowIdx >= 0 && rowIdx < Rows.Count)
                {
                    return Rows[rowIdx][colIdx];
                }
                else
                {
                    throw new Exception(GetType().Name + " row index error");
                }
            }
        }
        
        /// <summary>
        /// Wlasnosc umozliwiajaca zmiane i pobranie ilosci elementow listy
        /// zawierajacej elementy klasy Array
        /// </summary>
        /// <returns>Ilosc elementow listy</returns>
        public int RowCount
        {
            set
            {
                if (value < Rows.Count)
                {
                    Rows.RemoveRange(value, Rows.Count - value);
                }
                else
                {
                    while (value > Rows.Count)
                    {
                        Rows.Add(new Array(Size));
                    }
                }
            }
            
            get 
            { 
                return Rows.Count; 
            }
        }
        
        /// <summary>
        /// Wlasnosc tylko do odczytu umozliwiajaca pobranie rozmiaru tablic 
        /// klasy Array, ktora jest parametrem konstruktora klasy VTable
        /// </summary>
        /// <returns>Ilosc elementow tablicy</returns>
        public int ColumnCount
        {
            get 
            { 
                return Size; 
            }
        }

    }
}

