﻿using System.Drawing;

namespace VTableNamespace
{
    /// <summary>
    /// Pomocnicze typy okreslajace sposob prezentacji danych 
    /// przez metode Show w oparciu o strukture Record i klase Value
    /// </summary>
    public enum VTypes
    {
        /// <summary>
        /// Prezentuj dane uzywajac pojedynczych wierszy DataGridView
        /// i wlasnosci klasy Value AsInt
        /// </summary>
        Int,

        /// <summary>
        /// Prezentuj dane uzywajac podwojnych wierszy DataGridView
        /// i wlasnosci klasy Value AsStr, AsInt
        /// </summary>
        Rec
    }

    public partial class VTable
    {
        /// <summary>Metoda prezentujaca dane w komponencie DataGridView</summary>
        /// <param name="t">Typ okreslajacy sposob prezentacji</param>
        /// <param name="indexBase">
        /// Sposob numeracji wieszy, domyslnie od jeden, w przypadku typu VTypes.Rec co drugi wiersz
        /// </param>
        public void Show(VTypes t, int indexBase = 1)
        {
            if (Grid != null)
            {
                Grid.Rows.Clear();
                
                if (t == VTypes.Rec)
                {
                    Grid.RowCount = RowCount * 2;
                }
                else
                {
                    Grid.RowCount = RowCount;
                }
                
                Grid.ColumnCount = ColumnCount;

                for (int rowIdx = 0; rowIdx < Rows.Count; rowIdx++)
                {
                    for (int colIdx = 0; colIdx < ColumnCount; colIdx++)
                    {
                        if (t == VTypes.Int)
                        {
                            int i = this[colIdx, rowIdx].AsInt;

                            if (i == 0)
                            {
                                Grid[colIdx, rowIdx].Value = "";
                            }
                            else
                            {
                                Grid[colIdx, rowIdx].Value = i;
                            }
                            
                            Grid[colIdx, rowIdx].Style.BackColor = Color.Yellow;
                        }
                        else
                        {
                            Grid[colIdx, rowIdx * 2].Value = this[colIdx, rowIdx].AsStr;
                            
                            int i = this[colIdx, rowIdx].AsInt;

                            if (this[colIdx, rowIdx].AsInt == 0)
                            {
                                Grid[colIdx, rowIdx * 2 + 1].Value = "";
                            }
                            else
                            {
                                Grid[colIdx, rowIdx * 2 + 1].Value = i;
                            }
                            
                            Grid[colIdx, rowIdx * 2 + 1].Style.BackColor = Color.Yellow;
                        }

                        if (Grid.ColumnHeadersVisible)
                        {
                            Grid.Columns[colIdx].HeaderText = (colIdx + indexBase).ToString();
                        }
                    }

                    if (Grid.RowHeadersVisible)
                    {
                        if (t == VTypes.Rec)
                        {
                            Grid.Rows[rowIdx * 2].HeaderCell.Value = (rowIdx + indexBase).ToString();
                        }
                        else
                        {
                            Grid.Rows[rowIdx].HeaderCell.Value = (rowIdx + indexBase).ToString();
                        }
                    }
                }
            }
        }
    }
}


